export declare type Uint128 = string;
export declare type Address = string;
export declare type TokenType = CustomToken | NativeToken;
export declare type Decimal = string;
/**
 * Base64 encoded
 */
export declare type ViewingKey = string;
export declare class TokenPair {
    readonly token_0: TokenType;
    readonly token_1: TokenType;
    constructor(token_0: TokenType, token_1: TokenType);
}
export declare class TokenPairAmount {
    readonly pair: TokenPair;
    readonly amount_0: Uint128;
    readonly amount_1: Uint128;
    constructor(pair: TokenPair, amount_0: Uint128, amount_1: Uint128);
}
export declare class TokenTypeAmount {
    readonly token: TokenType;
    readonly amount: Uint128;
    constructor(token: TokenType, amount: Uint128);
}
export interface CustomToken {
    custom_token: {
        contract_addr: Address;
        token_code_hash: string;
    };
}
export interface NativeToken {
    native_token: {
        denom: string;
    };
}
export interface TokenInfo {
    name: string;
    symbol: string;
    decimals: number;
    total_supply?: Uint128 | null;
}
export interface Exchange {
    pair: TokenPair;
    address: Address;
}
export interface ExchangeSettings {
    sienna_burner?: Address | undefined;
    sienna_fee: ExchangeFee;
    swap_fee: ExchangeFee;
}
export interface ExchangeFee {
    denom: number;
    nom: number;
}
export interface PairInfo {
    amount_0: Uint128;
    amount_1: Uint128;
    factory: ContractInfo;
    liquidity_token: ContractInfo;
    pair: TokenPair;
    total_liquidity: Uint128;
}
export declare class TokenSaleConfig {
    /**
     * The token that will be used to buy the SNIP20.
     */
    readonly input_token: TokenType;
    /**
     * The total amount that each participant is allowed to buy.
     */
    readonly max_allocation: Uint128;
    /**
     * The minimum amount that each participant is allowed to buy.
     */
    readonly min_allocation: Uint128;
    /**
     * The maximum number of participants allowed.
     */
    readonly max_seats: number;
    /**
     * The price for a single token.
     */
    readonly rate: Uint128;
    readonly sold_token: ContractInfo;
    /**
     * The addresses that are eligible to participate in the sale.
     */
    readonly whitelist: Address[];
    constructor(
    /**
     * The token that will be used to buy the SNIP20.
     */
    input_token: TokenType, 
    /**
     * The total amount that each participant is allowed to buy.
     */
    max_allocation: Uint128, 
    /**
     * The minimum amount that each participant is allowed to buy.
     */
    min_allocation: Uint128, 
    /**
     * The maximum number of participants allowed.
     */
    max_seats: number, 
    /**
     * The price for a single token.
     */
    rate: Uint128, sold_token: ContractInfo, 
    /**
     * The addresses that are eligible to participate in the sale.
     */
    whitelist: Address[]);
}
export declare class Pagination {
    readonly start: number;
    /**
     * Max is 30.
     */
    readonly limit: number;
    constructor(start: number, 
    /**
     * Max is 30.
     */
    limit: number);
}
export declare enum TypeOfToken {
    Native = 0,
    Custom = 1
}
export declare function get_token_type(token: TokenType): TypeOfToken;
export interface Allowance {
    spender: Address;
    owner: Address;
    allowance: Uint128;
    expiration?: number | null;
}
export interface ExchangeRate {
    rate: Uint128;
    denom: string;
}
export interface RewardPool {
    lp_token: ContractInfo;
    /**
     * The reward amount allocated to this pool.
     */
    share: number;
    /**
     * Total amount locked by all participants.
     */
    size: number;
}
export interface PoolContractInfo {
    address: Address;
    pool: RewardPool;
}
export interface RewardsAccount {
    /**
     * The last time that the user claimed their rewards.
     */
    last_claimed: number;
    /**
     * The amount of LP tokens the owner has locked into this contract.
     */
    locked_amount: Uint128;
    /**
     * The owner of this account.
     */
    owner: Address;
    /**
     * A history of submitted tokens that aren't included in the rewards calculations yet.
     */
    pending_balances?: PendingBalance[] | null;
}
export interface PendingBalance {
    amount: Uint128;
    submitted_at: number;
}
export declare type ClaimError = {
    /**
     * Occurs when the rewards pool is currently empty.
     */
    type: "pool_empty";
} | {
    /**
     * Occurs when the user has no tokens locked in this pool.
     * In practice, this can occur when a wrong address was provided to the query.
     */
    type: "account_zero_locked";
} | {
    /**
     * It is possible for the user's share to be so little, that
     * the actual reward amount of rewards calculated to be zero.
     * However, it is highly unlikely in practice.
     */
    type: "account_zero_reward";
} | {
    /**
     * In Unix seconds.
     */
    time_to_wait: number;
    /**
     * Occurs when the user tries to claim earlier than the designated claim interval.
     */
    type: "early_claim";
};
export interface ClaimSimulationResult {
    /**
     * The actual amount of rewards that would be claimed.
     */
    actual_claimed: Uint128;
    error?: ClaimError | null;
    /**
     * The total amount of rewards that should be claimed.
     */
    reward_amount: Uint128;
    /**
     * The reward amount that would be claimed for a single portion.
     */
    reward_per_portion: Uint128;
}
export declare class ContractInfo {
    readonly code_hash: string;
    readonly address: Address;
    constructor(code_hash: string, address: Address);
}
export declare class ContractInstantiationInfo {
    readonly code_hash: string;
    readonly id: number;
    constructor(code_hash: string, id: number);
}
